package tech.zapt.zapt_sdk_flutter;

import android.app.Activity;
import android.app.ActivityManager;
import android.content.Context;

import android.os.PowerManager;
import android.util.Log;
import android.app.AlertDialog;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;

import tech.zapt.sdk.location.ZaptSDK;
import tech.zapt.sdk.location.ZaptSDKContext;
import tech.zapt.sdk.location.ZaptSDKOptions;
import tech.zapt.sdk.location.ZaptUserInfo;
import tech.zapt.sdk.location.beacon.Beacon;
import tech.zapt.sdk.location.beacon.BeaconListener;


public class ZaptSDKModule {

    private ZaptSDK zaptSDK;
    private final Context context;
    private Activity activity = null;

    @Nullable
    private static PowerManager.WakeLock sWakeLock;


    public ZaptSDKModule(Context context) {
        this.context = context;
        ZaptSDKOptions sdkOptions = ZaptSDKOptions.getInstance();

        sdkOptions.setForegroundBetweenScanPeriod(0);
        sdkOptions.setDisableSyncingForPositioning(true);

        this.zaptSDK = ZaptSDK.getInstance(this.context);
        this.zaptSDK.setBackground(false);
    }

    public void setActivity(Activity activity) {
        this.activity = activity;
    }

    public synchronized void initializeZaptSDK(String placeID) {
        if(!this.zaptSDK.isInitialized() && this.activity!=null) {
            this.zaptSDK.stop();
            this.zaptSDK.requestPermissions(this.activity);
            this.zaptSDK.initialize(placeID);
            setVisitable(placeID);
        }
    }

    public void stop(){
        if(this.zaptSDK != null) {
            this.zaptSDK.stop();
        }
    }

    public void setVisitable(String placeID){
        if(placeID != null) {
            this.zaptSDK.setVisitableId(placeID);
            ZaptUserInfo userInfo = ZaptUserInfo.getInstance(this.context);
            userInfo.setVisitableId(placeID);
            userInfo.commit();
        }
    }

    public String getMapLink (@NonNull String placeID, Map<String, String> options) {
        if (!placeID.isEmpty()) {
            initializeZaptSDK(placeID);
            String url;
            if(options == null || options.isEmpty()){
                url = this.zaptSDK.getMapLink();
            }else{
                url = this.zaptSDK.getMapLink(options);
            }
            return url;
        }
        return "";
    }

    public void requestPermissions(){
        try {
            this.zaptSDK.requestPermissions(this.activity);
        }catch (Exception error){
            Log.d("Error", error.getMessage());
        }
    }

    public void requestPermissionsBackground(Map<String, String> options) {
        try{
            String titleAlert = null, messageAlert = null, titleNoGranted = null, messageNoGranted = null;
            if(options != null) {
                titleAlert = options.get("titleAlert");
                messageAlert = options.get("messageAlert");
                titleNoGranted = options.get("titleNoGranted");
                messageNoGranted = options.get("messageNoGranted");
            }

            if(titleAlert == null) {
                titleAlert = "Esse app necessita de acesso a localização";
                messageAlert = "Para te apresentarmos notificações por proximidades necessitamos da permissão de localização sempre";

                titleNoGranted = "Funcionalidade limitada sem a permissão de localização";
                messageNoGranted = "Como a permissão de localização o tempo todo não foi concedida esse ap não vai te enviar as melhores notificações por proximidade. Para alterar a permissão acesse: Configurações -> Aplicativos -> Permissões e conceda a permissão total";
            }

            if(this.activity != null){
                final AlertDialog.Builder builderAlert = new AlertDialog.Builder(this.activity);
                if(titleAlert != null){
                    builderAlert.setTitle(titleAlert);
                }
                if(messageAlert != null){
                    builderAlert.setMessage(messageAlert);
                }

                final AlertDialog.Builder builderNotGranted = new AlertDialog.Builder(this.activity);
                if(titleNoGranted != null){
                    builderNotGranted.setTitle(titleNoGranted);
                }
                if(messageNoGranted != null){
                    builderNotGranted.setMessage(messageNoGranted);
                }

                this.zaptSDK.requestPermissionsBackground(this.activity, builderAlert, builderNotGranted);
            }
        }catch (Exception error){
            Log.d("Error", error.getMessage());
        }
    }
    
    public void addListener(String eventName, BinaryMessenger binaryMessenger) {
        try {
            EventChannel eventChannel = new EventChannel(binaryMessenger, eventName);
            eventChannel.setStreamHandler(new EventChannel.StreamHandler() {
                @Override
                public void onListen(Object arguments, EventChannel.EventSink events) {
                    ZaptSDKModule.this.zaptSDK.addBeaconListener(new BeaconListener() {
                        @Override
                        public void onScan(Collection<Beacon> collection) {
                            if (!isAppOnForeground(context)) {
                                return;
                            }

                            HashMap<String, Object> beaconMap;
                            ArrayList<Object> beaconsList = new ArrayList<>();
                            if (collection != null) {
                                for (Beacon b : collection) {
                                    beaconMap = new HashMap<>();
                                    beaconMap.put("uuid", b.getUuid());
                                    beaconMap.put("major", b.getMajor());
                                    beaconMap.put("minor", b.getMinor());
                                    beaconMap.put("rssi", b.getAvgRssi());
                                    beaconMap.put("accuracy", b.getDistance());
                                    beaconMap.put("txPower", b.getTxPower());
                                    beaconMap.put("macAddress", b.getMacAddress());
                                    beaconsList.add(beaconMap);
                                }
                            }
                            events.success(beaconsList);
                        }
                    });
                }

                @Override
                public void onCancel(Object arguments) {

                }
            });
        } catch (Exception error) {
            Log.d("Error", error.getMessage());
        }
    }

    public void removeListeners() {
        try{
            this.zaptSDK.removeBeaconListeners();
        } catch (Exception error){
            Log.d("Error", error.getMessage());
        }
	}

    public void setDebug(boolean debug){
        ZaptSDKOptions.getInstance().setDebug(debug);
    }

    private boolean isAppOnForeground(@NonNull Context context) {
        /**
         We need to check if app is in foreground otherwise the app will crash.
         http://stackoverflow.com/questions/8489993/check-android-application-is-in-foreground-or-not
         **/
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> appProcesses =
                activityManager.getRunningAppProcesses();
        if (appProcesses == null) {
            return false;
        }

        final String packageName = context.getPackageName();
        for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
            if (appProcess.importance ==
                    ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND &&
                    appProcess.processName.equals(packageName)) {
                return true;
            }
        }
        return false;
    }

    public void setDisableSyncingForPositioning(Boolean disable) {
        try{
            ZaptSDKContext sdkContext = ZaptSDKContext.getInstance(this.context);
            sdkContext.getOptions().setDisableSyncingForPositioning(disable);
        } catch (Exception error){
            Log.d("Error", error.getMessage());
        }
    }

    public void setDisableSyncingForAnalytics(Boolean disable) {
        try{
            ZaptSDKContext sdkContext = ZaptSDKContext.getInstance(this.context);
            sdkContext.getOptions().setDisableSyncingForAnalytics(disable);
        } catch (Exception error){
            Log.d("Error", error.getMessage());
        }
    }
}