import 'dart:io';
import 'package:zapt_sdk_flutter/zapt_sdk_flutter_controller.dart';

import 'package:zapt_sdk_flutter/entities/interest_point.dart';
import 'package:zapt_sdk_flutter/entities/map_coordinate_point.dart';
import 'zapt_sdk_flutter_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'dart:async';
import 'dart:convert';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:flutter/services.dart';
import 'zapt_webview_flutter.dart';

typedef OnChangeMapStatus = void Function(bool isLoading);

class ZaptSdkFlutter {
  static const stream = EventChannel("ReactNativeZaptSdkBeaconsFound");

  Future<String?> getMapLink(Map<String, dynamic> arguments) {
    return ZaptSdkFlutterPlatform.instance.getMapLink(arguments);
  }

  Future<void> requestPermissions() {
    return ZaptSdkFlutterPlatform.instance.requestPermissions();
  }

  Future<void> requestPermissionsBackground(Map<String, String> arguments) {
    return ZaptSdkFlutterPlatform.instance
        .requestPermissionsBackground(arguments);
  }

  Future<void> initialize(String placeId) {
    return ZaptSdkFlutterPlatform.instance.initialize(placeId);
  }

  Future<void> stop() {
    return ZaptSdkFlutterPlatform.instance.stop();
  }

  Future<void> setDisableSyncingForPositioning(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForPositioning(disable);
  }

  Future<void> setDisableSyncingForAnalytics(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForAnalytics(disable);
  }

  Future<void> addListener(String eventName) {
    return ZaptSdkFlutterPlatform.instance.addListener(eventName);
  }

  Future<void> removeLocationListener() {
    return ZaptSdkFlutterPlatform.instance.removeLocationListener();
  }

  Future<void> addBeaconListener(fn) async {
    await addListener("ReactNativeZaptSdkBeaconsFound");
    stream.receiveBroadcastStream().listen(fn);
  }
}

typedef OnCreated = void Function(ZaptSDKController? controller);

class ZaptMap extends StatefulWidget {
  const ZaptMap(
      {Key? key,
      required this.placeId,
      this.options,
      this.javascriptChannels,
      this.onChangeMapStatus,
      this.onCancelRoute,
      this.onCreated,
      this.onMapClick})
      : super(key: key);

  final String placeId;
  final Map<String, String>? options;
  final OnCreated? onCreated;
  final Set<JavascriptChannel>? javascriptChannels;
  final OnChangeMapStatus? onChangeMapStatus;
  final Function()? onCancelRoute;
  final Function(InterestPoint, MapCoordinatePoint, bool)? onMapClick;

  @override
  State<ZaptMap> createState() => _ZaptMapState();
}

class _ZaptMapState extends State<ZaptMap> {
  final _zaptSdkFlutterPlugin = ZaptSdkFlutter();
  ZaptWebViewController? _zaptWebViewController;
  WebViewController? _webViewController;
  String _mapLink = '';
  ZaptSDKController? _zaptSDKController;
  bool mapLoading = true;

  static const stream = EventChannel("ReactNativeZaptSdkBeaconsFound");

  @override
  void initState() {
    super.initState();
    initializeMap();
    initListenBeacons();
  }

  @override
  void didUpdateWidget(ZaptMap oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.placeId != widget.placeId ||
        !mapEquals(widget.options, oldWidget.options)) {
      initializeMap();
    }
  }

  Future<void> initializeMap() async {
    String mapLink;
    Map<String, String> mapOptions = widget.options ?? {};
    if (widget.onMapClick != null) {
      mapOptions['disablePopupOnClickPoi'] = 'true';
    }
    try {
      await _zaptSdkFlutterPlugin.initialize(widget.placeId);
      mapLink = await _zaptSdkFlutterPlugin.getMapLink({
            'placeId': widget.placeId,
            'options': mapOptions,
          }) ??
          "";
    } on PlatformException {
      mapLink = '';
    }
    if (mapLink.isNotEmpty) {
      setState(() {
        _mapLink = mapLink;
      });
      if (Platform.isAndroid) {
        _zaptWebViewController?.loadUrl(mapLink);
      } else if (Platform.isIOS) {
        _webViewController?.loadUrl(mapLink);
      }
    }
  }

  Future<void> initListenBeacons() async {
    await _zaptSdkFlutterPlugin.addListener("ReactNativeZaptSdkBeaconsFound");
    stream.receiveBroadcastStream().listen(_updateBeacons);
  }

  void _updateBeacons(beacons) {
    if (beacons.isNotEmpty) {
      String beaconsJSON = jsonEncode(beacons);
      String jsCode = 'window.onBeaconsFound($beaconsJSON);';
      if (Platform.isAndroid) {
        _zaptWebViewController?.runJavascript(jsCode);
      } else if (Platform.isIOS) {
        _webViewController?.runJavascript(jsCode);
      }
    }
  }

  Set<JavascriptChannel> getDefaultJavascriptChannels() {
    Set<JavascriptChannel>? jsChannels = {
      JavascriptChannel(
        name: "ZaptMapEventsOnMapLoad",
        onMessageReceived: (data) {
          if (widget.onChangeMapStatus != null) {
            widget.onChangeMapStatus!(data.message == "true");
          }
          setState(() {
            mapLoading = data.message == "true";
          });
        },
      ),
      JavascriptChannel(
        name: "ZaptMapEventsOnCancelRoute",
        onMessageReceived: (data) => {
          if (widget.onCancelRoute != null) {widget.onCancelRoute!()}
        },
      ),
    };

    if (widget.onMapClick != null) {
      jsChannels.add(
        JavascriptChannel(
          name: "ZaptMapEventsOnMapClick",
          onMessageReceived: (data) {
            Map<String, dynamic> dataMap = jsonDecode(data.message);
            bool clickInsidePOI = dataMap['clickInsidePOI'];
            MapCoordinatePoint clickedPoint =
                MapCoordinatePoint.fromJson(dataMap["clickedPoint"]);
            InterestPoint interestPoint =
                InterestPoint.fromJson(dataMap['interest']);
            if (widget.onMapClick != null) {
              widget.onMapClick!(interestPoint, clickedPoint, clickInsidePOI);
            }
          },
        ),
      );
    }
    return jsChannels;
  }

  onWebViewCreated(WebViewController? iosController,
      ZaptWebViewController? zaptWebViewController) {
    if (Platform.isAndroid) {
      setState(() {
        _zaptWebViewController = zaptWebViewController;
        _zaptSDKController = ZaptSDKController.forAndroid(
            androidWebViewController: _zaptWebViewController);
      });
    } else if (Platform.isIOS) {
      _webViewController = iosController;
      _zaptSDKController =
          ZaptSDKController.forIOS(iOSWebviewController: iosController);
    }

    if (widget.onChangeMapStatus != null) {
      widget.onChangeMapStatus!(true);
    }
  }

  void onPageFinished(String? url) {
    if (url != "about:blank" && url != null) {
      if (widget.onCreated != null) {
        widget.onCreated!(_zaptSDKController);
      }
    }
  }

  Widget getWebViewByPlatform() {
    Set<JavascriptChannel> jsChannels = getDefaultJavascriptChannels();
    if (widget.javascriptChannels != null) {
      jsChannels.addAll(widget.javascriptChannels!);
    }

    if (Platform.isAndroid) {
      return ZaptWebView(
        initialURL: _mapLink,
        javascriptChannels: jsChannels,
        onZaptWebViewCreated: (ZaptWebViewController controller) {
          onWebViewCreated(null, controller);
        },
        onPageFinishedCallback: onPageFinished,
      );
    } else if (Platform.isIOS) {
      return WebView(
        javascriptMode: JavascriptMode.unrestricted,
        javascriptChannels: jsChannels,
        onWebViewCreated: (WebViewController webViewController) {
          onWebViewCreated(webViewController, null);
        },
        gestureNavigationEnabled: true,
        onPageFinished: onPageFinished,
      );
    } else {
      throw UnsupportedError(
        "Package not supported by this platform. Current platform: $Platform._operatingSystem",
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(child: getWebViewByPlatform());
  }
}
