import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:zapt_sdk_flutter/zapt_webview_flutter.dart';

class ReferencePoint {
  ReferencePoint({
    required this.coordX,
    required this.coordY,
    required this.floor,
  });

  int coordX;
  int coordY;
  int floor;
}

class MapCenter {
  MapCenter({required this.x, required this.y, this.zoom});

  int x;
  int y;
  double? zoom;
}

class ZaptSDKController {
  WebViewController? _iOSWebviewController;
  ZaptWebViewController? _androidWebViewController;

  ZaptSDKController.forIOS({
    required iOSWebviewController,
  }) {
    _iOSWebviewController = iOSWebviewController;
  }

  ZaptSDKController.forAndroid({
    required androidWebViewController,
  }) {
    _androidWebViewController = androidWebViewController;
  }

  Future<bool> mapIsInitializing() async {
    String mapIsInitializing = "";
    if (_iOSWebviewController != null) {
      mapIsInitializing = await _iOSWebviewController!
          .runJavascriptReturningResult("zapt.initializing");
    }
    if (_androidWebViewController != null) {
      mapIsInitializing =
          await _androidWebViewController!.runJavascript("zapt.initializing");
    }
    return mapIsInitializing != 'false' && mapIsInitializing != '0';
  }

  Future<void> waitForMapInitialization() async {
    while (await mapIsInitializing()) {
      debugPrint("Awaiting map initialization");
      await Future.delayed(const Duration(seconds: 3));
    }
  }

  Future<void> _callJavascriptAPI<T>(String methodName, T args) async {
    String script = "api['$methodName']($args)";
    await waitForMapInitialization();
    await _iOSWebviewController?.runJavascript(script);
    await _androidWebViewController?.runJavascript(script);
  }

  // ignore: unused_element
  Future<String?> _callJavascriptAPIWithResult<T>(
      String methodName, T args) async {
    String script = "api[$methodName]($args)";
    debugPrint(script);
    if (_iOSWebviewController != null) {
      return await _iOSWebviewController!.runJavascriptReturningResult(script);
    }
    if (_androidWebViewController != null) {
      return await _androidWebViewController!.runJavascript(script);
    }
    return null;
  }

  Future<void> setFloor(int floorId) async {
    return await _callJavascriptAPI<int>("setFloor", floorId);
  }

  Future<void> setMapId(String mapId) async {
    return await _callJavascriptAPI<String>("setMapId", "$mapId, true");
  }

  Future<void> setCenter(MapCenter mapCenter) async {
    String args = "";
    if (mapCenter.zoom != null) {
      args = "${mapCenter.x},${mapCenter.y},${mapCenter.zoom}";
    } else {
      args = "${mapCenter.x},${mapCenter.y}";
    }
    return await _callJavascriptAPI<String>(
        "setCenter", "${mapCenter.x},${mapCenter.y},${mapCenter.zoom}");
  }

  Future<void> highlightInterestById(String poiId, bool openDetails) async {
    return await _callJavascriptAPI<String>(
        "highlightInterestById", "'$poiId', $openDetails");
  }

  Future<void> removeHighlightInterest(bool? resetMapView) async {
    return await _callJavascriptAPI<bool>(
        "removeHighlightInterest", resetMapView ?? false);
  }

  Future<void> setZoom(double zoom) async {
    return await _callJavascriptAPI<double>("setZoom", zoom);
  }

  Future<void> setRotation(double rotation) async {
    return await _callJavascriptAPI<double>("setRotation", rotation);
  }

  Future<void> createRouteByIds(String originId, String destinationId) async {
    return await _callJavascriptAPI<String>(
        "createRouteByIds", "'$originId', '$destinationId'");
  }

  Future<void> createRouteByCoordinates(
      ReferencePoint originPoint, ReferencePoint destinationPoint) async {
    String args =
        "[${originPoint.coordX}, ${originPoint.coordY}, ${originPoint.floor}],"
        "[${destinationPoint.coordX}, ${destinationPoint.coordY}, ${destinationPoint.floor}]";
    return await _callJavascriptAPI("createRouteByCoordinates", args);
  }

  Future<void> removeRoute() async {
    return await _callJavascriptAPI("removeRoute", "");
  }

  Future<void> resetMapView() async {
    return await _callJavascriptAPI("resetMapView", '');
  }
}
